/*
 * Aplicacin de ejemplo de uso de SQLite en C++
 * EjemploSQLite
 * Salvador Pozo, Con Clase (www.conclase.net)
 * Abril de 2012
 * Fichero: libro.cpp
 * fichero de implementacin para manipular datos de libros
 */

#include <iostream>
#include <iomanip>
#include "libro.h"
#include "editorial.h"
#include "autor.h"
#include "tema.h"

using namespace std;

void NuevoLibro(sqlite3 *db) {
    char titulo[64];
    char idioma[64];
    char formato[32];
    int editorial;
    sqlite3_stmt *ppStmt;
    int rc;
    char consulta[1024];
    bool existe, ignorar=false;
    char titulo2[64];
    int clave;
    char resp[2];

    for(int i= 0; i < 24; i++) cout << endl;
    cout << "A continuacion se pedira el titulo, idioma, formato y editorial del libro." << endl;

    cin.ignore();
    cout << "Titulo: ";
    cin.getline(titulo, 64);
    cout << "Idioma: ";
    cin.getline(idioma,64);
    cout << "Formato: ";
    cin.getline(formato,32);
    // Seleccionar editorial:
    editorial = ListaEditoriales(db);

    // Verificar si el nombre existe ya:
    sprintf(consulta, "SELECT clavelibro,titulo FROM libro WHERE titulo LIKE '%s';", titulo);
    rc = sqlite3_prepare_v2(db, consulta, -1, &ppStmt, NULL);
    existe=false;
    if( rc!=SQLITE_OK ){
        cout << "Error: " << sqlite3_errmsg(db) << endl;
    } else {
        if(SQLITE_ROW == sqlite3_step(ppStmt)) {
            existe = true;
            clave = sqlite3_column_int(ppStmt, 0);
            strncpy(titulo2, (const char*)sqlite3_column_text(ppStmt, 1), 64);
            titulo2[63]=0;
        }
        sqlite3_finalize(ppStmt);
    }

    if(!existe) {
        sprintf(consulta, "INSERT INTO libro(titulo,idioma,formato,claveeditorial) VALUES('%s','%s','%s',%d);",
                titulo, idioma, formato, editorial);
    } else {
        cout << "Ya existe un libro con el titulo " << titulo2 << " (s)obrescribir, insert(a)r o (i)gnorar: " << endl;
        cin >> resp;
        switch(resp[0]) {
            case 's':
                sprintf(consulta, "UPDATE libro SET titulo='%s',idioma='%s',formato='%s',claveeditorial=%d WHERE clavelibro=%d;",
                        titulo, idioma, formato, editorial, clave);
                break;
            case 'a':
                sprintf(consulta, "INSERT INTO libro(titulo,idioma,formato,claveeditorial) VALUES('%s','%s','%s','%d);",
                        titulo, idioma, formato, editorial);
                break;
            case 'i':
            default:
                ignorar=true;
        }
    }
    if(!ignorar) {
        if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
            cout << "Error: " << sqlite3_errmsg(db) << endl;
        }
        else cout << "Libro insertado" << endl;
    }
    cin.ignore();
}

int ListaLibros(sqlite3 *db) {
    sqlite3_stmt *ppStmt;
    int rc;
    char consulta[1024];
    int desplazamiento=0;
    char resp[10];
    bool salir=false;
    bool ultima;
    int fila=0;
    int i;

    // Mostrar una lista, teniendo en cuenta que puede haber ms de las que caben en una pantalla.
    do {
        cout << "Elegir libro" << endl << endl;
        sprintf(consulta, "SELECT clavelibro,titulo FROM libro ORDER BY titulo LIMIT 20 OFFSET %d;", desplazamiento);
        rc = sqlite3_prepare_v2(db, consulta, -1, &ppStmt, NULL);
        if( rc!=SQLITE_OK ){
            cout << "Error: " << sqlite3_errmsg(db) << endl;
        } else {
            i = 0;
            while(SQLITE_ROW == sqlite3_step(ppStmt)) {
                cout << sqlite3_column_int(ppStmt, 0) << ") " <<
                    sqlite3_column_text(ppStmt, 1) << endl;
                i++;
            }
            sqlite3_finalize(ppStmt);
        }
        ultima = (i < 20);
        while(i < 20) { cout << endl; i++; }
        cout << "\n" << "(n) editar, (s)ig pagina, (a)nt pagina, (x)salir" << endl;
        cin >> resp;
        switch(resp[0]) {
            case 's':
                if(!ultima) desplazamiento+=20;
                break;
            case 'a':
                if(desplazamiento > 0) desplazamiento-=20;
                break;
            case 'x':
                salir=true;
                break;
            default:
                if(isdigit(resp[0])) {
                    fila = atoi(resp);
                    salir=true;
                }
                break;
        }
    } while(!salir);

    return fila;
}

void EditarLibro(sqlite3 *db) {
    sqlite3_stmt *ppStmt;
    int rc;
    char consulta[1024];
    char titulo[64];
    char idioma[64];
    char formato[32];
    int editorial;
    int i;
    int fila;
    bool salir=true;

    fila = ListaLibros(db);

    // Editar:
    for(i = 0; i < 22; i++) cout << endl;
    sprintf(consulta, "SELECT titulo,idioma,formato,claveeditorial,editorial FROM libro NATURAL LEFT JOIN editorial WHERE clavelibro='%d';", fila);
    rc = sqlite3_prepare_v2(db, consulta, -1, &ppStmt, NULL);
    if( rc!=SQLITE_OK ){
        cout << "Error: " << sqlite3_errmsg(db) << endl;
    } else {
        i = 0;
        if(SQLITE_ROW == sqlite3_step(ppStmt)) {
            cout << "Titulo:    " << sqlite3_column_text(ppStmt, 0) << endl;
            cout << "Idioma:    " << sqlite3_column_text(ppStmt, 1) << endl;
            cout << "Formato:   " << sqlite3_column_text(ppStmt, 2) << endl;
            cout << "Editorial: " << sqlite3_column_text(ppStmt, 5) << endl;
            cout << "Dejar en blanco los campos que no se quieren modifiar" << endl;
            salir=false;
        }
        sqlite3_finalize(ppStmt);
    }
    if(!salir){
        cin.ignore();
        cout << "Titulo: ";
        cin.getline(titulo, 64);
        cout << "Idioma: ";
        cin.getline(idioma,64);
        cout << "Formato: ";
        cin.getline(formato,32);
        editorial = ListaEditoriales(db);
        if(strlen(titulo)>0) {
            sprintf(consulta, "UPDATE libro SET titulo='%s' WHERE clavelibro=%d;", titulo, fila);
            if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
                cout << "Error: " << sqlite3_errmsg(db) << endl;
            }
        }
        if(strlen(idioma)>0) {
            sprintf(consulta, "UPDATE libro SET idioma='%s' WHERE clavelibro=%d;", idioma, fila);
            if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
                cout << "Error: " << sqlite3_errmsg(db) << endl;
            }
        }
        if(strlen(formato)>0) {
            sprintf(consulta, "UPDATE libro SET formato='%s' WHERE clavelibro=%d;", formato, fila);
            if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
                cout << "Error: " << sqlite3_errmsg(db) << endl;
            }
        }
        if(editorial) {
            sprintf(consulta, "UPDATE libro SET claveeditorial=%d WHERE clavelibro=%d;", editorial, fila);
            if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
                cout << "Error: " << sqlite3_errmsg(db) << endl;
            }
        }
        cout << "Libro modificado" << endl;
        cin.ignore();
    }
}

void BorrarLibro(sqlite3 *db) {
    sqlite3_stmt *ppStmt;
    int rc;
    char consulta[1024];
    int fila;
    char resp[2];
    int i;
    bool salir=true;

    fila = ListaLibros(db);
    cout << "Borrar: " << fila << endl;

    for(i = 0; i < 22; i++) cout << endl;
    sprintf(consulta, "SELECT titulo,idioma,formato FROM editorial WHERE clavelibro='%d';", fila);
    rc = sqlite3_prepare_v2(db, consulta, -1, &ppStmt, NULL);
    if( rc!=SQLITE_OK ){
        cout << "Error: " << sqlite3_errmsg(db) << endl;
    } else {
        i = 0;
        if(SQLITE_ROW == sqlite3_step(ppStmt)) {
            cout << "Titulo:    " << sqlite3_column_text(ppStmt, 0) << endl;
            cout << "Idioma:    " << sqlite3_column_text(ppStmt, 1) << endl;
            cout << "Formato:   " << sqlite3_column_text(ppStmt, 2) << endl;
            salir=false;
        }
        sqlite3_finalize(ppStmt);
    }
    if(!salir){
        cin.ignore();
        cout << "Borrar este registro? (s/n)" << endl;
        cin >> resp;
        if(resp[0] == 's' || resp[0] == 'S') {
            sprintf(consulta, "DELETE FROM libro WHERE clavelibro=%d;", fila);
            if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
                cout << "Error: " << sqlite3_errmsg(db) << endl;
            }
        }
    }
    cout << "Libro borrado" << endl;
    cin.ignore();
}

void BuscarLibro(sqlite3 *db) {
    sqlite3_stmt *ppStmt;
    int rc;
    char consulta[1024];
    char filtro[256];
    char titulo[64];
    char idioma[64];
    char formato[32];
    char editorial[32];
    char autor[64];
    char tema[32];
    int i, colautor=5, coltema=5;

    for(i = 0; i < 22; i++) cout << endl;
    // Bsqueda de editoriales por nombre, direccion o telefono:
    cout << "Introducir cadenas de busqueda, _ para comodin de caracter, % para comodin de cadena" << endl;
    cout << "Dejar en blanco para ignorar el campo en la busqueda" << endl;
    cin.ignore();
    cout << "Titulo: ";
    cin.getline(titulo, 64);
    cout << "Idioma: ";
    cin.getline(idioma,64);
    cout << "Formato: ";
    cin.getline(formato,32);
    cout << "Editorial: ";
    cin.getline(editorial,32);
    cout << "Autor: ";
    cin.getline(autor,64);
    cout << "Tema: ";
    cin.getline(tema,32);

    if(strlen(titulo) == 0) strcpy(titulo, "%");
    if(strlen(idioma) == 0) strcpy(idioma, "%");
    if(strlen(formato) == 0) strcpy(formato, "%");
    if(strlen(editorial) == 0) strcpy(editorial, "%");
    strcpy(consulta, "SELECT titulo,idioma,formato,editorial");
    if(strlen(autor) > 0) { coltema++; strcat(consulta, ",autor"); }
    if(strlen(tema) > 0) strcat(consulta, ",tema");
    strcat(consulta, " FROM libro NATURAL LEFT JOIN editorial ");
    if(strlen(autor) > 0) strcat(consulta, "NATURAL JOIN escrito_por NATURAL JOIN autor ");
    if(strlen(tema)> 0) strcat(consulta, "NATURAL JOIN trata_sobre NATURAL JOIN tema ");
    sprintf(filtro, "WHERE titulo LIKE '%s' AND idioma LIKE '%s' AND formato LIKE '%s' "
            "AND editorial LIKE '%s'",
            titulo, idioma, formato, editorial);
    strcat(consulta, filtro);
    if(strlen(autor) > 0) {
        sprintf(filtro, " AND autor LIKE '%s'", autor);
        strcat(consulta, filtro);
    }
    if(strlen(tema) > 0) {
        sprintf(filtro, " AND tema LIKE '%s'", tema);
        strcat(consulta, filtro);
    }
    strcat(consulta, ";");

    rc = sqlite3_prepare_v2(db, consulta, -1, &ppStmt, NULL);
    if( rc!=SQLITE_OK ){
        cout << "Error: " << sqlite3_errmsg(db) << "\n" << consulta << endl;
    } else {
        i = 0;
        while(SQLITE_ROW == sqlite3_step(ppStmt)) {
            cout.setf(ios::left);
            cout.width(64);
            cout << sqlite3_column_text(ppStmt, 0) << endl;
            cout << sqlite3_column_text(ppStmt, 1) << " ";
            cout.width(32);
            cout << sqlite3_column_text(ppStmt, 2) << endl;
            cout.width(32);
            cout << sqlite3_column_text(ppStmt, 4) << endl;
            if(strlen(autor)>0) {
                cout.width(64);
                cout << sqlite3_column_text(ppStmt, colautor) << endl;
                i++;
            }
            if(strlen(tema)>0) {
                cout.width(32);
                cout << sqlite3_column_text(ppStmt, coltema) << endl;
                i++;
            }
            i +=3;
            if(i >= 21) {
                cout << "Pulsa return";
                cin.ignore();
                cin.get();
            }
        }
        sqlite3_finalize(ppStmt);
    }
    cin.ignore();
    cin.get();
}

void AnexarAutor(sqlite3 *db) {
    char consulta[1024];
    int clavelibro, claveautor;
    int i;

    for(i = 0; i < 22; i++) cout << endl;
    // Bsqueda de editoriales por nombre, direccion o telefono:
    cout << "Seleccionar un libro y un autor de las listas siguientes:" << endl;
    cin.ignore();
    clavelibro = ListaLibros(db);
    claveautor = ListaAutores(db);

    if(clavelibro && claveautor) {
        sprintf(consulta, "INSERT INTO escrito_por(clavelibro,claveautor) VALUES(%d,%d);", clavelibro, claveautor);
        if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
            cout << "Error: " << sqlite3_errmsg(db) << endl;
        }
        else cout << "Autor anexado a libro" << endl;
    }
}

void AnexarTema(sqlite3 *db) {
    char consulta[1024];
    int clavelibro, clavetema;
    int i;

    for(i = 0; i < 22; i++) cout << endl;
    // Bsqueda de editoriales por nombre, direccion o telefono:
    cout << "Seleccionar un libro y un tema de las listas siguientes:" << endl;
    cin.ignore();
    clavelibro = ListaLibros(db);
    clavetema = ListaTemas(db);

    if(clavelibro && clavetema) {
        sprintf(consulta, "INSERT INTO trata_sobre(clavelibro,clavetema) VALUES(%d,%d);", clavelibro, clavetema);
        if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
            cout << "Error: " << sqlite3_errmsg(db) << endl;
        }
        else cout << "Tema anexado a libro" << endl;
    }
}

int ListaAutoresLibro(sqlite3 *db, int clavelibro) {
    sqlite3_stmt *ppStmt;
    int rc;
    char consulta[1024];
    int desplazamiento=0;
    char resp[10];
    bool salir=false;
    bool ultima;
    int fila=0;
    int i;

    // Mostrar una lista, teniendo en cuenta que puede haber ms de las que caben en una pantalla.
    do {
        cout << "Elegir autor" << endl << endl;
        sprintf(consulta, "SELECT autor.claveautor,autor FROM escrito_por NATURAL JOIN autor "
                "WHERE clavelibro=%d ORDER BY autor LIMIT 20 OFFSET %d;", clavelibro, desplazamiento);
        rc = sqlite3_prepare_v2(db, consulta, -1, &ppStmt, NULL);
        if( rc!=SQLITE_OK ){
            cout << "Error: " << sqlite3_errmsg(db) << endl;
        } else {
            i = 0;
            while(SQLITE_ROW == sqlite3_step(ppStmt)) {
                cout << sqlite3_column_int(ppStmt, 0) << ") " <<
                    sqlite3_column_text(ppStmt, 1) << endl;
                i++;
            }
            sqlite3_finalize(ppStmt);
        }
        ultima = (i < 20);
        while(i < 20) { cout << endl; i++; }
        cout << "\n" << "(n) editar, (s)ig pagina, (a)nt pagina, (x)salir" << endl;
        cin >> resp;
        switch(resp[0]) {
            case 's':
                if(!ultima) desplazamiento+=20;
                break;
            case 'a':
                if(desplazamiento > 0) desplazamiento-=20;
                break;
            case 'x':
                salir=true;
                break;
            default:
                if(isdigit(resp[0])) {
                    fila = atoi(resp);
                    salir=true;
                }
                break;
        }
    } while(!salir);

    return fila;
}

void EliminarAutorLibro(sqlite3 *db) {
    char consulta[1024];
    int clavelibro, claveautor;
    int i;

    for(i = 0; i < 22; i++) cout << endl;
    // Bsqueda de editoriales por nombre, direccion o telefono:
    cout << "Seleccionar un libro:" << endl;
    cin.ignore();
    clavelibro = ListaLibros(db);
    if(clavelibro) {
        claveautor = ListaAutoresLibro(db, clavelibro);
        if(claveautor) {
            sprintf(consulta, "DELETE FROM escrito_por WHERE clavelibro=%d AND claveautor=%d;", clavelibro, claveautor);
            if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
                cout << "Error: " << sqlite3_errmsg(db) << endl;
            }
            else cout << "Autor eliminado de libro" << endl;
        }
    }
}

int ListaTemasLibro(sqlite3 *db, int clavelibro) {
    sqlite3_stmt *ppStmt;
    int rc;
    char consulta[1024];
    int desplazamiento=0;
    char resp[10];
    bool salir=false;
    bool ultima;
    int fila=0;
    int i;

    // Mostrar una lista, teniendo en cuenta que puede haber ms de las que caben en una pantalla.
    do {
        cout << "Elegir tema" << endl << endl;
        sprintf(consulta, "SELECT tema.clavetema,tema FROM trata_sobre NATURAL JOIN tema "
                "WHERE clavelibro=%d ORDER BY tema LIMIT 20 OFFSET %d;", clavelibro, desplazamiento);
        rc = sqlite3_prepare_v2(db, consulta, -1, &ppStmt, NULL);
        if( rc!=SQLITE_OK ){
            cout << "Error: " << sqlite3_errmsg(db) << endl;
        } else {
            i = 0;
            while(SQLITE_ROW == sqlite3_step(ppStmt)) {
                cout << sqlite3_column_int(ppStmt, 0) << ") " <<
                    sqlite3_column_text(ppStmt, 1) << endl;
                i++;
            }
            sqlite3_finalize(ppStmt);
        }
        ultima = (i < 20);
        while(i < 20) { cout << endl; i++; }
        cout << "\n" << "(n) editar, (s)ig pagina, (a)nt pagina, (x)salir" << endl;
        cin >> resp;
        switch(resp[0]) {
            case 's':
                if(!ultima) desplazamiento+=20;
                break;
            case 'a':
                if(desplazamiento > 0) desplazamiento-=20;
                break;
            case 'x':
                salir=true;
                break;
            default:
                if(isdigit(resp[0])) {
                    fila = atoi(resp);
                    salir=true;
                }
                break;
        }
    } while(!salir);

    return fila;
}

void EliminarTemaLibro(sqlite3 *db) {
    char consulta[1024];
    int clavelibro, clavetema;
    int i;

    for(i = 0; i < 22; i++) cout << endl;
    // Bsqueda de editoriales por nombre, direccion o telefono:
    cout << "Seleccionar un libro:" << endl;
    cin.ignore();
    clavelibro = ListaLibros(db);
    if(clavelibro) {
        clavetema = ListaTemasLibro(db, clavelibro);
        if(clavetema) {
            sprintf(consulta, "DELETE FROM trata_sobre WHERE clavelibro=%d AND clavetema=%d;", clavelibro, clavetema);
            if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
                cout << "Error: " << sqlite3_errmsg(db) << endl;
            }
            else cout << "Tema eliminado de libro" << endl;
        }
    }
}
