/*
 * Aplicacin de ejemplo de uso de SQLite en C++
 * EjemploSQLite
 * Salvador Pozo, Con Clase (www.conclase.net)
 * Marzo de 2012
 * Fichero: main.cpp
 * fichero principal
 * Incluir en el enlazador la librera libsqlite.a (sqlite)
 * Facilitar el acceso a la librera de enlace dinmico sqlite3.dll
 */

#include <iostream>
#include <sqlite/sqlite3.h>
#include "menus.h"
#include "editorial.h"
#include "autor.h"
#include "tema.h"
#include "libro.h"
#include "ejemplar.h"
#include "socio.h"
#include "prestamo.h"

const int nTablas = 9;

bool VerificarTablas(sqlite3 *);

using namespace std;

int main()
{
    int rc;
    sqlite3 *db;
    int nivel=1;
    bool salir;

    // Abrir base de datos
    rc = sqlite3_open("biblioteca.db", &db);
    if(SQLITE_OK != rc) {
        cout << "Error: No se puede abrir la base de datos" << endl;
		return 1;
    }

    if(!VerificarTablas(db)) return -1;
    if(!IniciarMenu(db)) return -1;

    do {
        MostrarMenu(db, nivel);
        switch(LeerMenu(db, nivel)) {
            case ABRIRMENU:
               // Nada que hacer.
               break;
            case NUEVAEDITORIAL:
               NuevaEditorial(db);
               break;
            case EDITAREDITORIAL:
               EditarEditorial(db);
               break;
            case BORRAREDITORIAL:
               BorrarEditorial(db);
               break;
            case CONSULTAEDITORIAL:
               BuscarEditorial(db);
               break;
            case NUEVOAUTOR:
               NuevoAutor(db);
               break;
            case EDITARAUTOR:
               EditarAutor(db);
               break;
            case BORRARAUTOR:
               BorrarAutor(db);
               break;
            case CONSULTAAUTOR:
               BuscarAutor(db);
               break;
            case NUEVOTEMA:
               NuevoTema(db);
               break;
            case EDITARTEMA:
               EditarTema(db);
               break;
            case BORRARTEMA:
               BorrarTema(db);
               break;
            case CONSULTATEMA:
               BuscarTema(db);
               break;
            case NUEVOLIBRO:
               NuevoLibro(db);
               break;
            case EDITARLIBRO:
               EditarLibro(db);
               break;
            case ANEXARAUTOR:
               AnexarAutor(db);
               break;
            case ANEXARTEMA:
               AnexarTema(db);
               break;
            case ELIMINARAUTOR:
               EliminarAutorLibro(db);
               break;
            case ELIMINARTEMA:
               EliminarTemaLibro(db);
               break;
            case BORRARLIBRO:
               BorrarLibro(db);
               break;
            case CONSULTALIBRO:
               BuscarLibro(db);
               break;
            case NUEVOEJEMPLAR:
               NuevoEjemplar(db);
               break;
            case EDITAREJEMPLAR:
               EditarEjemplar(db);
               break;
            case BORRAREJEMPLAR:
               BorrarEjemplar(db);
               break;
            case CONSULTAEJEMPLAR:
               BuscarEjemplar(db);
               break;
            case NUEVOSOCIO:
               NuevoSocio(db);
               break;
            case EDITARSOCIO:
               EditarSocio(db);
               break;
            case BORRARSOCIO:
               BorrarSocio(db);
               break;
            case CONSULTASOCIO:
               BuscarSocio(db);
               break;
            case NUEVOPRESTAMO:
               NuevoPrestamo(db);
               break;
            case EDITARPRESTAMO:
               CambiarFechaPrestamo(db);
               break;
            case ANULARPRESTAMO:
               AnularPrestamo(db);
               break;
            case DEVOLUCION:
               DevolucionPrestamo(db);
               break;
            case HISTORIALLIBRO:
               HistorialLibro(db);
               break;
            case HISTORIALSOCIO:
               HistorialSocio(db);
               break;
            case CONSULTAPRESTAMOS:
               ConsultaPrestamos(db);
               break;
            case SALIR:
               salir = true;
        }
    } while(!salir);

    // Cerrar base de datos
    sqlite3_close(db);
    return 0;
}

bool VerificarTablas(sqlite3 *db) {
    char consulta[36];
    char *tabla[] = {
        "editorial",
        "libro",
        "autor",
        "tema",
        "ejemplar",
        "socio",
        "prestamo",
        "trata_sobre",
        "escrito_por"
    };
    char *create[] = {
        "CREATE TABLE editorial("
          "claveeditorial INTEGER PRIMARY KEY,"
          "editorial TEXT,"
          "direccion TEXT,"
          "telefono TEXT);",
        "CREATE TABLE libro("
          "clavelibro INTEGER PRIMARY KEY,"
          "titulo TEXT,"
          "idioma TEXT,"
          "formato TEXT,"
          "claveeditorial INTEGER "
          "REFERENCES editorial(claveeditorial) "
          "ON DELETE SET NULL "
          "ON UPDATE CASCADE);",
        "CREATE TABLE autor("
          "claveautor INTEGER PRIMARY KEY,"
          "autor TEXT);",
        "CREATE TABLE tema("
          "clavetema INTEGER PRIMARY KEY,"
          "tema TEXT);",
        "CREATE TABLE ejemplar("
          "claveejemplar INTEGER PRIMARY KEY,"
          "clavelibro INTEGER "
          "REFERENCES libro(clavelibro) "
          "ON DELETE CASCADE "
          "ON UPDATE CASCADE,"
          "numeroorden INTEGER NOT NULL,"
          "edicion INTEGER,"
          "ubicacion TEXT,"
          "categoria TEXT,"
          "UNIQUE(numeroorden,clavelibro));",
        "CREATE TABLE socio("
          "clavesocio INTEGER PRIMARY KEY,"
          "socio TEXT,"
          "direccion TEXT,"
          "telefono TEXT,"
          "categoria TEXT);",
        "CREATE TABLE prestamo("
          "clavesocio INTEGER "
          "REFERENCES socio(clavesocio) "
          "ON DELETE SET NULL "
          "ON UPDATE CASCADE,"
          "claveejemplar INTEGER "
          "REFERENCES ejemplar(claveejemplar) "
          "ON DELETE SET NULL "
          "ON UPDATE CASCADE,"
          "fecha_prestamo DATE NOT NULL,"
          "fecha_devolucion DATE DEFAULT NULL,"
          "notas TEXT);",
        "CREATE TABLE trata_sobre("
          "clavelibro INTEGER NOT NULL "
          "REFERENCES libro(clavelibro) "
          "ON DELETE CASCADE "
          "ON UPDATE CASCADE,"
          "clavetema INTEGER NOT NULL "
          "REFERENCES tema(clavetema) "
          "ON DELETE CASCADE "
          "ON UPDATE CASCADE,"
          "UNIQUE(clavelibro,clavetema));",
        "CREATE TABLE escrito_por("
          "clavelibro INTEGER NOT NULL "
          "REFERENCES libro(clavelibro) "
          "ON DELETE CASCADE "
          "ON UPDATE CASCADE,"
          "claveautor INTEGER NOT NULL "
          "REFERENCES autor(claveautor) "
          "ON DELETE CASCADE "
          "ON UPDATE CASCADE,"
          "UNIQUE(clavelibro,claveautor));"
    };

    // Activar soporte para claves forneas
    if(SQLITE_OK != sqlite3_exec(db, "PRAGMA foreign_keys = ON;", 0, 0, 0)) {
        cout << "Imposible activar claves foraneas" << endl;
        return false;
    }

    for(int i = 0; i < nTablas; i++) {
        sprintf(consulta, "SELECT COUNT(*) FROM %s;", tabla[i]);
        if(SQLITE_OK != sqlite3_exec(db, consulta, 0, 0, 0)) {
            cout << "La tabla " << tabla[i] << " no existe." << endl;
            if(SQLITE_OK != sqlite3_exec(db, create[i], 0, 0, 0)) {
               cout << "Error al crear la tabla " << tabla[i] << endl;
               return false;
            }
        }
    }
    return true;
}
